-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.ButtonBar = {}

function XMacros.ButtonBar.Create(setName, barName, parentFrame, positionFrame, barSpec, tooltip, mouseOverBar, pressedCallback)
	local buttonBar = {}

	buttonBar.setName = setName
	buttonBar.barName = barName
	buttonBar.parentFrame = parentFrame
	buttonBar.positionFrame = positionFrame
	buttonBar.tooltip = tooltip
	buttonBar.pressedCallback = pressedCallback
	buttonBar.barSpec = barSpec
	buttonBar.mouseRefCount = 0
	buttonBar.mouseOverBar = mouseOverBar
	buttonBar.editButtons = {}
	buttonBar.frame = UI.CreateFrame("Frame", "XMacros.ButtonBar", parentFrame)
	buttonBar.frame:SetSecureMode("restricted")
	buttonBar.frame:SetVisible(false)
	buttonBar.frame:SetPoint("TOPLEFT", buttonBar.positionFrame, "TOPLEFT", buttonBar.barSpec.x, buttonBar.barSpec.y)
	
	XMacros.ButtonBar.CreateButtons(buttonBar)
	
	return buttonBar
end

function XMacros.ButtonBar.CreateButtons(buttonBar)
	buttonBar.margin = math.floor(buttonBar.barSpec.textureSize / 16)
	buttonBar.buttonSize = buttonBar.barSpec.textureSize
	buttonBar.width = buttonBar.buttonSize * buttonBar.barSpec.cols
	buttonBar.height = buttonBar.buttonSize * buttonBar.barSpec.rows
	buttonBar.frame:SetWidth(buttonBar.width)
	buttonBar.frame:SetHeight(buttonBar.height)
	
	buttonBar.buttons = {}
	for row = 1, buttonBar.barSpec.rows do
		for col = 1, buttonBar.barSpec.cols do
			local xOffset = buttonBar.buttonSize * (col - 1)
			local yOffset = buttonBar.buttonSize * (row - 1)
			local pressedFunc = nil
			if buttonBar.pressedCallback then
				pressedFunc = function() buttonBar.pressedCallback(buttonBar, row, col) end
			end
			
			local mouseInFunc = nil
			if buttonBar.mouseOverBar == true then
				mouseInFunc = function(increment) XMacros.ButtonBar.MouseInBarCount(buttonBar, increment) end
			end
			
			local name = XMacros.ButtonBar.GetButtonName(row, col)
			local button = XMacros.Button.Create(buttonBar.frame, name, xOffset, yOffset, buttonBar.buttonSize, buttonBar.buttonSize, buttonBar.margin, buttonBar.tooltip, pressedFunc, mouseInFunc)
			local buttonSpec = buttonBar.barSpec.buttonSpecs[name]
			if buttonSpec and buttonSpec.macros then
				for macroName, macro in pairs(buttonSpec.macros) do
					if buttonBar.pressedCallback then
						XMacros.Button.AddMacro(button, macroName, macro.category, macro.macro)
					end
				end
			elseif buttonBar.pressedCallback then
				XMacros.Button.AddMacro(button, "Left") 
			end
			
			buttonBar.buttons[name] = button
		end
	end
	
	for buttonName, buttonSpec in pairs(buttonBar.barSpec.buttonSpecs) do
		local row, col = XMacros.ButtonBar.GetButtonPosition(buttonName)
		if XMacros.ButtonBar.IsPositionValid(buttonBar, row, col) == false then
			buttonBar.barSpec.buttonSpecs[buttonName] = nil
		else
			if buttonSpec.textureName and buttonSpec.textureAddon then
				XMacros.ButtonBar.SetTexture(buttonBar, row, col, buttonSpec.textureAddon, buttonSpec.textureName)
			end

			if buttonSpec.abilityName then
				XMacros.ButtonBar.SetAbilityName(buttonBar, row, col, buttonSpec.abilityName)
			end

			if buttonSpec.macros then
				for macroName, macro in pairs(buttonSpec.macros) do
					if buttonBar.editCallback then
						XMacros.ButtonBar.AddMacro(buttonBar, row, col, macroName, macro.category, macro.macro)
					end
				end
			elseif buttonBar.editCallback then
				XMacros.ButtonBar.AddMacro(buttonBar, row, col, "Left")
			end
		end
	end
end

function XMacros.ButtonBar.IsPositionValid(buttonBar, row, col)
	if row < 1 or row > buttonBar.barSpec.rows then
		return false
	end
	if col < 1 or col > buttonBar.barSpec.cols then
		return false
	end
	
	return true
end

function XMacros.ButtonBar.GetButton(buttonBar, row, col)
	if row < 1 or row > buttonBar.barSpec.rows then
		print("Cannot get button from row " .. tostring(row) .. " must be in range 1-" .. buttonBar.barSpec.rows)
		return
	end
	if col < 1 or col > buttonBar.barSpec.cols then
		print("Cannot get button from col " .. tostring(col) .. " must be in range 1-" .. buttonBar.barSpec.cols)
		return
	end
	
	return buttonBar.buttons[XMacros.ButtonBar.GetButtonName(row, col)]
end

function XMacros.ButtonBar.GetPostion(buttonBar)
	return buttonBar.barSpec.x, buttonBar.barSpec.y
end

function XMacros.ButtonBar.SetTexture(buttonBar, row, col, textureAddon, textureName)
	XMacros.Button.SetTexture(XMacros.ButtonBar.GetButton(buttonBar, row, col), textureAddon, textureName)
end

function XMacros.ButtonBar.SetAbilityName(buttonBar, row, col, abilityName)
	XMacros.Button.SetAbilityName(XMacros.ButtonBar.GetButton(buttonBar, row, col), abilityName)
end

function XMacros.ButtonBar.AddMacro(buttonBar, row, col, buttonName, category, macroName, macro)
	XMacros.Button.AddMacro(XMacros.ButtonBar.GetButton(buttonBar, row, col), buttonName, category, macroName, macro)
end

function XMacros.ButtonBar.SetVisible(buttonBar, visible, showAll)
	if visible == false then
		buttonBar.mouseRefCount = 0
	elseif buttonBar.dragFrame then
		buttonBar.mouseRefCount = 1
	end
	
	if visible == true and buttonBar.dragFrame then
		showAll = true
	end

	if visible == false or showAll == false then
		for _, frame in ipairs(buttonBar.editButtons) do
			frame:SetVisible(false)
		end
		
		buttonBar.editButtons = {}
	end

	local editButtonSize = 16
	if buttonBar.barSpec.textureSize < 40 then
		if buttonBar.barSpec.textureSize < 20 then
			editButtonSize = 8
		else
			editButtonSize = 12
		end
	end
	
	for row = 1, buttonBar.barSpec.rows do
		for col = 1, buttonBar.barSpec.cols do
			local button = XMacros.ButtonBar.GetButton(buttonBar, row, col)
			if button then
				if visible == true then
					if showAll == true or XMacros.Button.HasTexture(button) then
						XMacros.Button.SetVisible(button, true)
					else
						XMacros.Button.SetVisible(button, false)
					end
					
					if showAll == true then
						if col > 1 then
							local prevButton = XMacros.ButtonBar.GetButton(buttonBar, row, col - 1)
							if XMacros.Button.HasTexture(button) or XMacros.Button.HasTexture(prevButton) then
								local editButton = UI.CreateFrame("SimpleTextureButtonExt", "Edit_" .. row .. "_" .. col, buttonBar.parentFrame)
								editButton:SetLayer(buttonBar.parentFrame:GetLayer() + 1)
								editButton:SetTexture("XMacros", "Textures/LeftRight.png")
								editButton:SetPoint("CENTER", XMacros.Button.GetFrame(prevButton), "CENTERRIGHT")
								editButton:SetHeight(editButtonSize)
								editButton:SetWidth(editButtonSize)
								editButton.Event.LeftPress = function() XMacros.ButtonBar.SwapButtons(buttonBar, prevButton, button) end
								table.insert(buttonBar.editButtons, editButton)
							end
						end
						
						if row > 1 then
							local prevButton = XMacros.ButtonBar.GetButton(buttonBar, row - 1, col)
							if XMacros.Button.HasTexture(button) or XMacros.Button.HasTexture(prevButton) then
								local editButton = UI.CreateFrame("SimpleTextureButtonExt", "Edit_" .. row .. "_" .. col, buttonBar.parentFrame)
								editButton:SetTexture("XMacros", "Textures/UpDown.png")
								editButton:SetLayer(buttonBar.parentFrame:GetLayer() + 1)
								editButton:SetPoint("CENTER", XMacros.Button.GetFrame(prevButton), "BOTTOMCENTER")
								editButton:SetHeight(editButtonSize)
								editButton:SetWidth(editButtonSize)
								editButton.Event.LeftPress = function() XMacros.ButtonBar.SwapButtons(buttonBar, prevButton, button) end
								table.insert(buttonBar.editButtons, editButton)
							end
						end
						
						if row == 1 and buttonBar.barSpec.cols > 1 then
							local editButton = UI.CreateFrame("SimpleTextureButtonExt", "Delete_" .. row .. "_" .. col, buttonBar.parentFrame)
							editButton:SetTexture("XMacros", "Textures/Delete.png")
							editButton:SetLayer(buttonBar.parentFrame:GetLayer() + 1)
							editButton:SetPoint("BOTTOMLEFT", XMacros.Button.GetFrame(button), "TOPLEFT")
							editButton:SetHeight(editButtonSize)
							editButton:SetWidth(editButtonSize)
							editButton.Event.LeftPress = function() XMacros.ButtonBar.DeleteColumn(buttonBar, col) end
							editButton.Event.MouseIn = function() XMacros.ButtonBar.SelectColumn(buttonBar, col, true) end
							editButton.Event.MouseOut = function() XMacros.ButtonBar.SelectColumn(buttonBar, col, false) end
							table.insert(buttonBar.editButtons, editButton)
						end

						if col == buttonBar.barSpec.cols and buttonBar.barSpec.rows > 1 then
							local editButton = UI.CreateFrame("SimpleTextureButtonExt", "Delete_" .. row .. "_" .. col, buttonBar.parentFrame)
							editButton:SetTexture("XMacros", "Textures/Delete.png")
							editButton:SetLayer(buttonBar.parentFrame:GetLayer() + 1)
							editButton:SetPoint("BOTTOMLEFT", XMacros.Button.GetFrame(button), "BOTTOMRIGHT")
							editButton:SetHeight(editButtonSize)
							editButton:SetWidth(editButtonSize)
							editButton.Event.LeftPress = function() XMacros.ButtonBar.DeleteRow(buttonBar, row) end
							editButton.Event.MouseIn = function() XMacros.ButtonBar.SelectRow(buttonBar, row, true) end
							editButton.Event.MouseOut = function() XMacros.ButtonBar.SelectRow(buttonBar, row, false) end
							table.insert(buttonBar.editButtons, editButton)
						end
						
						if col == buttonBar.barSpec.cols and row == 1 then
							local editButton = UI.CreateFrame("SimpleTextureButtonExt", "Add_" .. row .. "_" .. col, buttonBar.parentFrame)
							editButton:SetTexture("XMacros", "Textures/Plus.png")
							editButton:SetLayer(buttonBar.parentFrame:GetLayer() + 1)
							editButton:SetPoint("TOPLEFT", XMacros.Button.GetFrame(button), "TOPRIGHT")
							editButton:SetHeight(editButtonSize)
							editButton:SetWidth(editButtonSize)
							editButton.Event.LeftPress = function() XMacros.ButtonBar.AddColumn(buttonBar) end
							table.insert(buttonBar.editButtons, editButton)

							local editButton = UI.CreateFrame("SimpleTextureButtonExt", "Config_" .. row .. "_" .. col, buttonBar.parentFrame)
							editButton:SetTexture("XMacros", "Textures/Wrench.png")
							editButton:SetLayer(buttonBar.parentFrame:GetLayer() + 1)
							editButton:SetPoint("BOTTOMRIGHT", XMacros.Button.GetFrame(button), "TOPRIGHT")
							editButton:SetHeight(editButtonSize)
							editButton:SetWidth(editButtonSize)
							editButton.Event.LeftPress = function() XMacros.ButtonBar.ConfigBar(buttonBar) end
							table.insert(buttonBar.editButtons, editButton)
						end
						
						if col == buttonBar.barSpec.cols and row == buttonBar.barSpec.rows then
							local editButton = UI.CreateFrame("SimpleTextureButtonExt", "Add_" .. row .. "_" .. col, buttonBar.parentFrame)
							editButton:SetTexture("XMacros", "Textures/Plus.png")
							editButton:SetLayer(buttonBar.parentFrame:GetLayer() + 1)
							editButton:SetPoint("TOPRIGHT", XMacros.Button.GetFrame(button), "BOTTOMRIGHT")
							editButton:SetHeight(editButtonSize)
							editButton:SetWidth(editButtonSize)
							editButton.Event.LeftPress = function() XMacros.ButtonBar.AddRow(buttonBar) end
							table.insert(buttonBar.editButtons, editButton)
						end
					end
				else
					XMacros.Button.SetVisible(button, false)
				end
			end
		end
	end
	
	if buttonBar.dragFrame and visible == false then
		XMacros.ButtonBar.SetDragEnabled(buttonBar, false)
	end
	
	XMacros.ButtonBar.SetAlpha(buttonBar)
	buttonBar.frame:SetVisible(visible)
end

function XMacros.ButtonBar.SwapButtons(buttonBar, prevButton, button)
	for _, frame in ipairs(buttonBar.editButtons) do
		frame:SetVisible(false)
		frame.Event.LeftPress = nil
	end
	
	buttonBar.editButtons = {}

	local prevName = XMacros.Button.GetName(prevButton)
	local nextName = XMacros.Button.GetName(button)
	XMacros.Button.SetName(button, prevName)
	XMacros.Button.SetName(prevButton, nextName)
	
	local prevSpec = buttonBar.barSpec.buttonSpecs[prevName]
	local nextSpec = buttonBar.barSpec.buttonSpecs[nextName]
	buttonBar.barSpec.buttonSpecs[prevName] = nextSpec
	buttonBar.barSpec.buttonSpecs[nextName] = prevSpec
	
	local x = prevButton.x
	local y = prevButton.y
	XMacros.Button.SetPosition(prevButton, button.x, button.y)
	XMacros.Button.SetPosition(button, x, y)
	buttonBar.buttons[prevName] = button
	buttonBar.buttons[nextName] = prevButton
	
	XMacros.ButtonBar.SetVisible(buttonBar, true, true)
	XMacros.Button.SetHighlightVisible(prevButton, false)
	XMacros.Button.SetHighlightVisible(button, false)
end

function XMacros.ButtonBar.SetDragEnabled(buttonBar, enabled, movedCallback)
	if enabled == false then
		if buttonBar.dragFrame then
			buttonBar.dragFrame.frame.Event.RightUp = nil
			buttonBar.dragFrame.frame.Event.RightUpoutside = nil
			buttonBar.dragFrame.frame.Event.RightDown = nil
			buttonBar.dragFrame.frame.Event.MouseMove = nil
			buttonBar.dragFrame.frame = nil
			buttonBar.dragFrame = nil
		end
	elseif not buttonBar.dragFrame then
		local dragFrame = {}

		dragFrame.dragging = false
		dragFrame.frame = UI.CreateFrame("Frame", "DragFrame", buttonBar.frame)
		dragFrame.frame:SetSecureMode("restricted")
		dragFrame.frame:SetAllPoints(buttonBar.frame)
		dragFrame.frame:SetLayer(buttonBar.frame:GetLayer() + 1)
		dragFrame.frame:SetMouseMasking("limited")
		dragFrame.frame.Event.RightUp = function () XMacros.ButtonBar.MouseUp(buttonBar) end
		dragFrame.frame.Event.RightUpoutside = function () XMacros.ButtonBar.MouseUp(buttonBar) end
		dragFrame.frame.Event.RightDown = function () XMacros.ButtonBar.MouseDown(buttonBar) end
		dragFrame.frame.Event.MouseMove = function (event, x, y) XMacros.ButtonBar.MouseMove(buttonBar, x, y) end
		dragFrame.movedCallback = movedCallback
		buttonBar.dragFrame = dragFrame
	end
end

function XMacros.ButtonBar.MouseDown(buttonBar)
	if buttonBar.dragFrame.dragging == false then
		local mouse = Inspect.Mouse()
		buttonBar.dragFrame.xOffset = buttonBar.barSpec.x - mouse.x
		buttonBar.dragFrame.yOffset = buttonBar.barSpec.y - mouse.y
		buttonBar.dragFrame.dragging = true
	end
end

function XMacros.ButtonBar.MouseMove(buttonBar, x, y)
	if buttonBar.dragFrame.dragging == true then
		local totalWidth = buttonBar.positionFrame:GetWidth()
		local totalHeight = buttonBar.positionFrame:GetHeight()
		buttonBar.barSpec.x = x + buttonBar.dragFrame.xOffset
		buttonBar.barSpec.y = y + buttonBar.dragFrame.yOffset
		if buttonBar.barSpec.x < 0 then buttonBar.barSpec.x = 0 end
		if buttonBar.barSpec.y < 0 then buttonBar.barSpec.y = 0 end
		if buttonBar.barSpec.x + buttonBar.width > totalWidth then buttonBar.barSpec.x = totalWidth - buttonBar.width end
		if buttonBar.barSpec.y + buttonBar.height > totalHeight then buttonBar.barSpec.y = totalHeight - buttonBar.height end
		buttonBar.frame:SetPoint("TOPLEFT", buttonBar.positionFrame, "TOPLEFT", buttonBar.barSpec.x, buttonBar.barSpec.y)
	end
end

function XMacros.ButtonBar.MouseUp(buttonBar)
	buttonBar.dragFrame.dragging = false
	if buttonBar.dragFrame.movedCallback then
		buttonBar.dragFrame.movedCallback(buttonBar.barSpec.x, buttonBar.barSpec.y)
	end
end

function XMacros.ButtonBar.SetAlpha(buttonBar)
	local alpha = 1
	if buttonBar.mouseOverBar == true then
		if buttonBar.mouseRefCount <= 0 then
			if buttonBar.mouseRefCount < 0 then print ("Error ref count=" .. buttonBar.mouseRefCount) buttonBar.mouseRefCount = 0 end
			alpha = 0
		else
			alpha = 1
		end
	end
	
	for row = 1, buttonBar.barSpec.rows do
		for col = 1, buttonBar.barSpec.cols do
			local button = XMacros.ButtonBar.GetButton(buttonBar, row, col)
			if button then
				XMacros.Button.SetAlpha(button, alpha)
			end
		end
	end
end

function XMacros.ButtonBar.MouseInBarCount(buttonBar, increment)
	if buttonBar.dragFrame then
		return
	end
	
	buttonBar.mouseRefCount = buttonBar.mouseRefCount + increment
	XMacros.ButtonBar.SetAlpha(buttonBar)
end

function XMacros.ButtonBar.GetButtonName(row, col)
	return tostring(row) .. "," .. tostring(col)
end

function XMacros.ButtonBar.GetButtonPosition(buttonName)
	local nameParts = XMacros.Utils.Split(buttonName, ',')
	return tonumber(nameParts[1]), tonumber(nameParts[2])
end

function XMacros.ButtonBar.SelectRow(buttonBar, row, selected)
	for col = 1, buttonBar.barSpec.cols do
		local button = XMacros.ButtonBar.GetButton(buttonBar, row, col)
		if button then
			if selected == true then
				XMacros.Button.SetAlpha(button, 0.3)
			else
				XMacros.Button.SetAlpha(button, 1)
			end
		end
	end
end

function XMacros.ButtonBar.SelectColumn(buttonBar, col, selected)
	for row = 1, buttonBar.barSpec.rows do
		local button = XMacros.ButtonBar.GetButton(buttonBar, row, col)
		if button then
			if selected == true then
				XMacros.Button.SetAlpha(button, 0.3)
			else
				XMacros.Button.SetAlpha(button, 1)
			end
		end
	end
end

function XMacros.ButtonBar.RedrawButtons(buttonBar)
	for _, frame in ipairs(buttonBar.editButtons) do
		frame:SetVisible(false)
		frame.Event.LeftPress = nil
	end
	
	buttonBar.editButtons = {}

	for _, button in pairs(buttonBar.buttons) do
		XMacros.Button.SetVisible(button, false)
	end
	
	buttonBar.buttons = {}
	
	XMacros.ButtonBar.CreateButtons(buttonBar)
	XMacros.ButtonBar.SetVisible(buttonBar, true, true)
end

function XMacros.ButtonBar.DeleteRow(buttonBar, row)
	if buttonBar.barSpec.rows < 2 then
		return
	end
	
	for col = 1, buttonBar.barSpec.cols do
		local buttonName = XMacros.ButtonBar.GetButtonName(row, col)
		buttonBar.barSpec.buttonSpecs[buttonName] = nil
	end

	for newRow = row + 1, buttonBar.barSpec.rows do
		for col = 1, buttonBar.barSpec.cols do
			local buttonName = XMacros.ButtonBar.GetButtonName(newRow, col)
			local newButtonName = XMacros.ButtonBar.GetButtonName(newRow - 1, col)
			buttonBar.barSpec.buttonSpecs[newButtonName] = buttonBar.barSpec.buttonSpecs[buttonName]
			buttonBar.barSpec.buttonSpecs[buttonName] = nil
		end
	end
	
	buttonBar.barSpec.rows = buttonBar.barSpec.rows - 1

	XMacros.ButtonBar.RedrawButtons(buttonBar)
end

function XMacros.ButtonBar.DeleteColumn(buttonBar, col)
	if buttonBar.barSpec.cols < 2 then
		return
	end
	
	for row = 1, buttonBar.barSpec.rows do
		local buttonName = XMacros.ButtonBar.GetButtonName(row, col)
		buttonBar.barSpec.buttonSpecs[buttonName] = nil
	end

	for newCol = col + 1, buttonBar.barSpec.cols do
		for row = 1, buttonBar.barSpec.rows do
			local buttonName = XMacros.ButtonBar.GetButtonName(row, newCol)
			local newButtonName = XMacros.ButtonBar.GetButtonName(row, newCol - 1)
			buttonBar.barSpec.buttonSpecs[newButtonName] = buttonBar.barSpec.buttonSpecs[buttonName]
			buttonBar.barSpec.buttonSpecs[buttonName] = nil
		end
	end
	
	buttonBar.barSpec.cols = buttonBar.barSpec.cols - 1

	XMacros.ButtonBar.RedrawButtons(buttonBar)
end

function XMacros.ButtonBar.AddRow(buttonBar)
	buttonBar.barSpec.rows = buttonBar.barSpec.rows + 1
	XMacros.ButtonBar.RedrawButtons(buttonBar)
end

function XMacros.ButtonBar.AddColumn(buttonBar)
	buttonBar.barSpec.cols = buttonBar.barSpec.cols + 1
	XMacros.ButtonBar.RedrawButtons(buttonBar)
end

function XMacros.ButtonBar.ConfigBar(buttonBar)
end
