-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.SetEditor = {}
XMacros.SetEditor.name = "XMacros.SetEditor"
local SET_DESCRIPTION = "Bar Sets are collections of button bars.  Each character role can be configured to show many bar sets.  This allows you to put button bars that are shared between roles into one bar set and then role specific button bars into another bar set.\n\nBar Set names can only contain letters and numbers.\n\n"

--
-- Main functions
--

--
-- UI functions
--
local function RefreshScreen(setEditor)
	local selectedSet = setEditor.setSelect:GetSelectedItem()
	if not selectedSet then
		setEditor.setRename:SetEnabled(false)
		setEditor.setDelete:SetEnabled(false)
	else
		setEditor.setRename:SetEnabled(true)
		setEditor.setDelete:SetEnabled(true)
	end

	local setList = {}
	local role = setEditor.roleSelect:GetSelectedItem()
	if role ~= nil then
		setList = XMacros.ButtonSets.GetRoleSet(setEditor.buttonSets, role)
		if not setList then
			setList = {}
		end
	end
	
	setEditor.roleList:SetItems(setList)
end

local function RefreshRole(setEditor, role)
	setEditor.refreshCallback(role)
end

local function GetDefaultSetName()
	return Inspect.Unit.Detail("player").calling:gsub("^%l", string.upper) .. "Default"
end

local function RefreshSetList(setEditor)
	setEditor.setSelect:SetSelectedItem(nil)
	setEditor.setSelect:SetItems(XMacros.ButtonSets.GetSetNames(setEditor.buttonSets))
end

local function AddSetCallback(setEditor, setName)
	XMacros.SetEditor.SetEnabled(setEditor, true)
	
	if setName ~= nil then
		XMacros.ButtonSets.AddSet(setEditor.buttonSets, setName)
	end

	RefreshSetList(setEditor)
	RefreshScreen(setEditor)
end

local function AddSet(setEditor)
	XMacros.SetEditor.SetEnabled(setEditor, false)
	XMacros.EntryWindow.ShowEntryWindow(setEditor.entryWindow, SET_DESCRIPTION .. "Enter new bar set name",
						setName, function(x, newSetName) AddSetCallback(setEditor, newSetName) end, "[^%a%d]+")
end

local function RenameSetCallback(setEditor, newSetName)
	XMacros.SetEditor.SetEnabled(setEditor, true)
	
	local setName = setEditor.setSelect:GetSelectedItem()
	if newSetName ~= nil and setName ~= nil and newSetName ~= setName then
		XMacros.ButtonSets.RenameSet(setEditor.buttonSets, setName, newSetName)
	end

	RefreshSetList(setEditor)
	RefreshScreen(setEditor)
end

local function RenameSet(setEditor)
	local setName = setEditor.setSelect:GetSelectedItem()
	if setName then
		XMacros.SetEditor.SetEnabled(setEditor, false)
		XMacros.EntryWindow.ShowEntryWindow(setEditor.entryWindow, SET_DESCRIPTION .. "Enter new name for bar set " .. setName, setName, function(x, newSetName) RenameSetCallback(setEditor, newSetName) end, "[^%a%d]+")
	end
end

local function DeleteSetCallback(setEditor, ok)
	if ok == true then
		XMacros.ButtonSets.DeleteSet(setEditor.buttonSets, setEditor.setSelect:GetSelectedItem())
		setEditor.setSelect:SetItems(XMacros.ButtonSets.GetSetNames(setEditor.buttonSets))
	end

	RefreshSetList(setEditor)
	RefreshScreen(setEditor)
end

local function DeleteSet(setEditor)
	XMacros.ConfirmationWindow.ShowConfirmationWindow(setEditor.cfmWindow, "Do you really want to delete bar set " .. setEditor.setSelect:GetSelectedItem() .. "?", function(x, ok) DeleteSetCallback(setEditor, ok) end)
end

local function ChangeRoleSets(setEditor, role, setList, checklist)
	XMacros.SetEditor.SetEnabled(setEditor, true)

	if checklist then
		local newSetList = {}
		for i, name in ipairs(setList) do
			if checklist[i] then
				table.insert(newSetList, name)
			end
		end
		
		XMacros.ButtonSets.SetRoleSet(setEditor.buttonSets, role, newSetList)
		RefreshRole(setEditor, role)
		RefreshSetList(setEditor)
		RefreshScreen(setEditor)
	end
end

local function ChangeSets(setEditor)
	local role = setEditor.roleSelect:GetSelectedItem()
	if role == nil then
		return
	end
	
	local setList = XMacros.ButtonSets.GetRoleSet(setEditor.buttonSets, role)
	if not setList then
		setList = {}
	end
	
	local setDictionary = {}
	for _, name in ipairs(setList) do
		setDictionary[name] = 1
	end
	
	local setList = XMacros.ButtonSets.GetSetNames(setEditor.buttonSets)
	if #setList < 1 then
		table.insert(setList, GetDefaultSetName())
	end
	
	local checklist = {}
	for row, name in ipairs(setList) do
		if setDictionary[name] then
			table.insert(checklist, true)
		else
			table.insert(checklist, false)
		end
	end
		
	XMacros.SetEditor.SetEnabled(setEditor, false)
	XMacros.TextPicker.Show(setEditor.txtMultiPicker, "Choose Sets for role " .. role, setList, function(a, checks) ChangeRoleSets(setEditor, role, setList, checks) end, checklist)
end

local function AddBarSetCallback(a, setEditor, setName, row, col, icon, combatShow)
	setEditor.windowFrame:SetVisible(true)
	if a then
		local role = setEditor.roleSelect:GetSelectedItem()
		if role == nil then
			return
		end

		local roleSets = XMacros.ButtonSets.GetRoleSetNames(setEditor.buttonSets, role)
		if roleSets == nil or roleSets == "" then
			XMacros.ButtonSets.SetRoleSet(setEditor.buttonSets, role, setName)
			RefreshSetList(setEditor)
			RefreshScreen(setEditor)
		end
		
		XMacros.ButtonSets.AddBar(setEditor.buttonSets, setName, 100, 100, row, col, icon, combatShow)
		setEditor.refreshCallback(role)
	end
end

local function AddBarSetChosen(setEditor, setName)
	if setName then
		XMacros.AddBarEditor.Show(setEditor.addBarWindow, function(a, row, col, icon, combatShow) AddBarSetCallback(a, setEditor, setName, row, col, icon, combatShow) end)
	else
		setEditor.windowFrame:SetVisible(true)
	end
end

local function AddBar(setEditor)
	local setList = {}
	local role = setEditor.roleSelect:GetSelectedItem()
	if role ~= nil then
		setList = XMacros.ButtonSets.GetRoleSet(setEditor.buttonSets, role)
		if not setList or #setList < 1 then
			setList = {}
			table.insert(setList, GetDefaultSetName())
		end
	end

	setEditor.windowFrame:SetVisible(false)
	if #setList > 1 then
		XMacros.TextPicker.Show(setEditor.txtPicker, "Choose Set for the new Bar", setList, function(a, setName) AddBarSetChosen(setEditor, setName) end)
	else
		AddBarSetChosen(setEditor, setList[1])
	end
end

local function CreateRolePane(setEditor)
	local roleChange = UI.CreateFrame("RiftButton", setEditor.roleContent:GetName() .. "Change", setEditor.roleContent)
	roleChange:SetText("Change Sets")
	roleChange:SetPoint("BOTTOMLEFT", setEditor.roleContent, "BOTTOMLEFT", 5, -3)
	roleChange.Event.LeftPress = function() ChangeSets(setEditor) end
	setEditor.roleChange = roleChange
	
	local addBar = UI.CreateFrame("RiftButton", setEditor.roleContent:GetName() .. "AddBar", setEditor.roleContent)
	addBar:SetText("Add Bar")
	addBar:SetPoint("BOTTOMLEFT", roleChange, "TOPLEFT", 0, -5)
	addBar.Event.LeftPress = function() AddBar(setEditor) end
	setEditor.addBar = addBar
	
	local roleLabel = UI.CreateFrame("Text", setEditor.roleContent:GetName() .. "Label", setEditor.roleContent)
	roleLabel:SetPoint("TOPLEFT", setEditor.roleContent, "TOPLEFT", 5, 3)
	roleLabel:SetText("Role")

	local roleSelect = UI.CreateFrame("SimpleSelectExt", setEditor.roleContent:GetName() .. "Select", setEditor.roleContent)
	roleSelect:SetLayer(1)
	roleSelect:SetItems({"1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15", "16", "17", "18", "19", "20"})
	roleSelect:SetPoint("TOPLEFT", roleLabel, "TOPRIGHT", 5, 0)
	roleSelect:SetPoint("RIGHT", roleChange, "RIGHT")
	roleSelect:SetHeight(roleLabel:GetHeight())
	roleSelect:SetBorder(1, 0.5, 0.5, 0.5, 1)
	roleSelect.Event.ItemSelect = function(x, role) RefreshRole(setEditor, role) RefreshScreen(setEditor) end
	setEditor.roleSelect = roleSelect
	
	local roleList = UI.CreateFrame("SimpleScrollListExt", setEditor.roleContent:GetName() .. "List", setEditor.roleContent)
	roleList:SetPoint("TOPLEFT", roleSelect, "TOPRIGHT", 10, 0)
	roleList:SetPoint("TOPRIGHT", setEditor.roleContent, "TOPRIGHT", -10, 3)
	roleList:SetBorder(1, 0.5, 0.5, 0.5, 1)
	setEditor.roleList = roleList
end

local function SetEnabledRolePane(setEditor, enabled)
	setEditor.roleChange:SetEnabled(enabled)
	setEditor.roleSelect:SetEnabled(enabled)
	setEditor.roleList:SetEnabled(enabled)
end

local function CreateSetPane(setEditor)
	local setAdd = UI.CreateFrame("RiftButton", setEditor.setContent:GetName() .. "Add", setEditor.setContent)
	setAdd:SetText("Add Set")
	setAdd:SetPoint("BOTTOMLEFT", setEditor.setContent, "BOTTOMLEFT", 5, -3)
	setAdd.Event.LeftPress = function() AddSet(setEditor) end
	setEditor.setAdd = setAdd
	
	local setRename = UI.CreateFrame("RiftButton", setEditor.setContent:GetName() .. "Rename", setEditor.setContent)
	setRename:SetText("Rename Set")
	setRename:SetPoint("TOPRIGHT", setEditor.setContent, "TOPRIGHT", -5, 0)
	setRename.Event.LeftPress = function() RenameSet(setEditor) end
	setEditor.setRename = setRename
	
	local setDelete = UI.CreateFrame("RiftButton", setEditor.setContent:GetName() .. "Delete", setEditor.setContent)
	setDelete:SetText("Delete Set")
	setDelete:SetPoint("BOTTOMRIGHT", setEditor.setContent, "BOTTOMRIGHT", -5, -3)
	setDelete.Event.LeftPress = function() DeleteSet(setEditor) end
	setEditor.setDelete = setDelete
	
	local setLabel = UI.CreateFrame("Text", setEditor.setContent:GetName() .. "Label", setEditor.setContent)
	setLabel:SetPoint("TOPLEFT", setEditor.setContent, "TOPLEFT", 5, 4)
	setLabel:SetText("Bar Set")

	local setSelect = UI.CreateFrame("SimpleSelectExt", setEditor.setContent:GetName() .. "Select", setEditor.setContent)
	setSelect:SetLayer(1)
	setSelect:SetPoint("TOPLEFT", setLabel, "TOPRIGHT", 5, 0)
	setSelect:SetPoint("TOPRIGHT", setRename, "TOPLEFT", -5, 4)
	setSelect:SetHeight(setLabel:GetHeight())
	setSelect:SetBorder(1, 0.5, 0.5, 0.5, 1)
	setSelect.Event.ItemSelect = function() RefreshScreen(setEditor) end
	setEditor.setSelect = setSelect
end

local function SetEnabledSetPane(setEditor, enabled)
	setEditor.setAdd:SetEnabled(enabled)
	setEditor.setRename:SetEnabled(enabled)
	setEditor.setDelete:SetEnabled(enabled)
	setEditor.setSelect:SetEnabled(enabled)
end

local function CreateMacroPane(setEditor)
	local showMacros = UI.CreateFrame("RiftButton", setEditor.macroContent:GetName() .. "Add", setEditor.macroContent)
	showMacros:SetText("Macros")
	showMacros:SetPoint("BOTTOMLEFT", setEditor.macroContent, "BOTTOMLEFT", 5, -math.ceil((setEditor.macroContent:GetHeight() - showMacros:GetHeight()) / 2) - 3)
	showMacros.Event.LeftPress = function() XMacros.EditMacros() end
	setEditor.showMacros = showMacros
end

function XMacros.SetEditor.Create(parentFrame, x, y, buttonSets, refreshCallback, closeCallback, movedCallback)
	local setEditor = {}
	setEditor.parentFrame = parentFrame

	setEditor.buttonSets = buttonSets
	setEditor.closeCallback = closeCallback
	setEditor.refreshCallback = refreshCallback
	setEditor.movedCallback = movedCallback
	setEditor.width = 430
	setEditor.height = 400
	
	--if x >= 0 then
	--	setEditor.x = x
	--	setEditor.y = y
	--else
		setEditor.x = math.ceil(UIParent:GetWidth() / 2)
		setEditor.y = math.ceil(UIParent:GetHeight() / 2 - setEditor.height / 2)
	--end
	
	setEditor.windowFrame = UI.CreateFrame("SimpleWindow", "XMacros", parentFrame)
	setEditor.windowFrame:SetSecureMode("normal")
	setEditor.windowFrame:SetCloseButtonVisible(true)
	setEditor.windowFrame:SetVisible(false)
	setEditor.windowFrame:SetTitle("XMacros")
	setEditor.windowFrame:SetWidth(setEditor.width)
	setEditor.windowFrame:SetHeight(setEditor.height)
	setEditor.windowFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", setEditor.x, setEditor.y)
	setEditor.windowFrame.Event.Close = function() XMacros.SetEditor.WindowClose(setEditor, true) end

	local xMargin = 15
	setEditor.roleBorder = UI.CreateFrame("SimpleBorderExt", setEditor.windowFrame:GetName() .. "RoleBorder", setEditor.windowFrame)
	setEditor.roleBorder:SetLayer(3)
	setEditor.roleBorder:SetPoint("TOPLEFT", setEditor.windowFrame, "TOPLEFT", xMargin, 50)
	setEditor.roleBorder:SetPoint("BOTTOMRIGHT", setEditor.windowFrame, "TOPRIGHT", -xMargin, 205)
	local content = UI.CreateFrame("Frame", setEditor.roleBorder:GetName() .. "Content", setEditor.roleBorder)
	setEditor.roleBorder:SetContent(content)
	setEditor.roleContent = setEditor.roleBorder:GetContent()
	
	setEditor.setBorder = UI.CreateFrame("SimpleBorderExt", setEditor.windowFrame:GetName() .. "SetBorder", setEditor.windowFrame)
	setEditor.setBorder:SetLayer(2)
	setEditor.setBorder:SetPoint("TOPLEFT", setEditor.roleBorder, "BOTTOMLEFT", 0, 5)
	setEditor.setBorder:SetPoint("BOTTOMRIGHT", setEditor.roleBorder, "BOTTOMRIGHT", 0, 100)
	local content = UI.CreateFrame("Frame", setEditor.setBorder:GetName() .. "Content", setEditor.setBorder)
	setEditor.setBorder:SetContent(content)
	setEditor.setContent = setEditor.setBorder:GetContent()
	
	setEditor.macroBorder = UI.CreateFrame("SimpleBorderExt", setEditor.windowFrame:GetName() .. "MacroBorder", setEditor.windowFrame)
	setEditor.macroBorder:SetLayer(1)
	setEditor.macroBorder:SetPoint("TOPLEFT", setEditor.setBorder, "BOTTOMLEFT", 0, 5)
	setEditor.macroBorder:SetPoint("BOTTOMRIGHT", setEditor.windowFrame, "BOTTOMRIGHT", -xMargin, -15)
	local content = UI.CreateFrame("Frame", setEditor.macroBorder:GetName() .. "Content", setEditor.macroBorder)
	setEditor.macroBorder:SetContent(content)
	setEditor.macroContent = setEditor.macroBorder:GetContent()
	
	CreateRolePane(setEditor)
	CreateSetPane(setEditor)
	CreateMacroPane(setEditor)
	
	setEditor.txtPicker = XMacros.TextPicker.Create(setEditor.parentFrame, setEditor.windowFrame)
	setEditor.txtMultiPicker = XMacros.TextPicker.Create(setEditor.parentFrame, setEditor.windowFrame, true)
	setEditor.addBarWindow = XMacros.AddBarEditor.Create(setEditor.parentFrame, setEditor.windowFrame)
	setEditor.wrnWindow = XMacros.WarningWindow.CreateWarningWindow(setEditor.parentFrame, setEditor.windowFrame)
	setEditor.cfmWindow = XMacros.ConfirmationWindow.CreateConfirmationWindow(setEditor.parentFrame, setEditor.windowFrame)
	setEditor.entryWindow = XMacros.EntryWindow.CreateEntryWindow(setEditor.parentFrame, setEditor.windowFrame)
	
	return setEditor
end

function XMacros.SetEditor.WindowClose(setEditor, okPressed)
	setEditor.windowFrame:SetVisible(false)
	if setEditor.closeCallback then
		setEditor.closeCallback(setEditor, okPressed)
	end
end

function XMacros.SetEditor.Show(setEditor, currentRole)
	setEditor.roleSelect:SetSelectedItem(currentRole)
	RefreshRole(setEditor, currentRole)
	RefreshSetList(setEditor)
	RefreshScreen(setEditor)
	setEditor.windowFrame:SetVisible(true)
end

function XMacros.SetEditor.SetEnabled(setEditor, enabled)
	setEditor.windowFrame.closeButton:SetEnabled(enabled)
	SetEnabledRolePane(setEditor, enabled)
	SetEnabledSetPane(setEditor, enabled)
end
