-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.Timer = {}
XMacros.Timer.name = "XMacros.Timer"
XMacros.Timer.eventsByTime = {}
XMacros.Timer.eventsByID = {}
XMacros.Timer.eventID = 0
XMacros.Timer.currentTime = 0

--
-- Main functions
--

function XMacros.Timer.AddEvent()
	table.insert(Event.System.Update.Begin, XMacros.Timer.timerHook)
end

function XMacros.Timer.RemoveEvent()
	for i, hook in ipairs(Event.System.Update.Begin) do
		if hook == XMacros.Timer.timerHook then
			table.remove(Event.System.Update.Begin, i)
			return
		end
	end
end

function XMacros.Timer.Clear()
	XMacros.Timer.eventsByTime = {}
	XMacros.Timer.eventsByID = {}
	XMacros.Timer.RemoveEvent()
	XMacros.Timer.currentTime = 0
end

function XMacros.Timer.AddCallback(name, duration, callback, repeatIt)
	if next(XMacros.Timer.eventsByID) == nil then
		XMacros.Timer.AddEvent()
	end
	
	if XMacros.Timer.currentTime == 0 then
		XMacros.Timer.currentTime = XMacros.Timer.InternalGetTime()
	end
	
	local eventID = XMacros.Timer.InternalGetID()
	local roundedDuration = XMacros.Utils.Round(duration, 1)
	if roundedDuration < 0.1 then
		return nil
	end
	
	XMacros.Timer.InternalAdd(eventID, name, roundedDuration, callback, repeatIt)
	return eventID
end

function XMacros.Timer.RemoveCallback(timerId)
	local thisXMacrosTimer = XMacros.Timer.eventsByID[timerId]
	if thisXMacrosTimer then
		local dueTimeKey = XMacros.Timer.GetTimeKey(thisXMacrosTimer.dueTime)
		XMacros.Timer.eventsByID[timerId] = nil
		XMacros.Timer.eventsByTime[dueTimeKey][timerId] = nil

		local deleteTable = true
		for i, j in pairs(XMacros.Timer.eventsByTime[dueTimeKey]) do
			deleteTable = false
		end

		if deleteTable == true then
			XMacros.Timer.eventsByTime[dueTimeKey] = nil
		end
	end
end

function XMacros.Timer.InternalAdd(id, name, duration, callback, repeatIt)
	local dueTime = XMacros.Timer.currentTime + (duration * 10)
	
	local thisXMacrosTimer = {}
	thisXMacrosTimer.id = id
	thisXMacrosTimer.name = name
	thisXMacrosTimer.duration = duration
	thisXMacrosTimer.callback = callback
	thisXMacrosTimer.dueTime = dueTime
	if (repeatIt == true) then
		thisXMacrosTimer.repeatXMacrosTimer = true
	else
		thisXMacrosTimer.repeatXMacrosTimer = false
	end
	
	local dueTimeKey = XMacros.Timer.GetTimeKey(dueTime)
	if not XMacros.Timer.eventsByTime[dueTimeKey] then
		XMacros.Timer.eventsByTime[dueTimeKey] = {}
	end

	XMacros.Timer.eventsByTime[dueTimeKey][id] = thisXMacrosTimer
	XMacros.Timer.eventsByID[id] = thisXMacrosTimer
end

function XMacros.Timer.InternalGetTime()
	return math.floor(Inspect.Time.Frame() * 10)
end

function XMacros.Timer.GetTime(currentTime)
	return currentTime / 10
end

function XMacros.Timer.GetCurrentTime()
	if XMacros.Timer.currentTime == 0 then
		return XMacros.Timer.GetTime(XMacros.Timer.InternalGetTime())
	else
		return XMacros.Timer.GetTime(XMacros.Timer.currentTime)
	end
end

function XMacros.Timer.InternalGetID()
	XMacros.Timer.eventID = XMacros.Timer.eventID + 1
	return "#" .. XMacros.Timer.eventID
end

function XMacros.Timer.GetTimeKey(thisTime)
	return "@" .. thisTime
end


--
-- Event Handlers
--

function XMacros.Timer.Refresh()
	local newTime = XMacros.Timer.InternalGetTime()
	
	if next(XMacros.Timer.eventsByID) == nil then
		XMacros.Timer.Clear()
	elseif newTime ~= XMacros.Timer.currentTime then
		local nextXMacrosTimerTime = XMacros.Timer.currentTime + 1
		for currentTime = nextXMacrosTimerTime, newTime do
			XMacros.Timer.currentTime = currentTime
			local timeKey = XMacros.Timer.GetTimeKey(XMacros.Timer.currentTime)
			if XMacros.Timer.eventsByTime[timeKey] then
				for id, thisXMacrosTimer in pairs(XMacros.Timer.eventsByTime[timeKey]) do
					if thisXMacrosTimer.repeatXMacrosTimer == true then
						XMacros.Timer.InternalAdd(thisXMacrosTimer.id, thisXMacrosTimer.name, thisXMacrosTimer.duration, thisXMacrosTimer.callback, thisXMacrosTimer.repeatXMacrosTimer)
					else
						XMacros.Timer.eventsByID[thisXMacrosTimer.id] = nil
					end

					thisXMacrosTimer.callback(XMacros.Timer.GetTime(currentTime))
				end

				XMacros.Timer.eventsByTime[timeKey] = nil
			end
		end
	end
end

XMacros.Timer.timerHook = { XMacros.Timer.Refresh, "XMacros", "XMacros.Timer" }
