-- There is no copyright on this code

-- Permission is hereby granted, free of charge, to any person obtaining a copy of this software and
-- associated documentation files (the "Software"), to deal in the Software without restriction, including
-- without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
-- copies of the Software, and to permit persons to whom the Software is furnished to do so.

-- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
-- LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
-- NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
-- WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
-- SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

--
-- Global variables
--
local addon, XMacros = ...
XMacros.Tooltip = {}
XMacros.Tooltip.name = "XMacros.Tooltip"

--
-- Main functions
--

--
-- UI functions
--

function XMacros.Tooltip.Create(parentFrame, positonFrame, x, y, positionChangedCallback)
	local tooltip = {}
	tooltip.x = x
	tooltip.y = y
	tooltip.positionChangedCallback = positionChangedCallback
	tooltip.windowFrame = UI.CreateFrame("Texture", "tooltip", parentFrame)
	tooltip.windowFrame:SetVisible(false)
	tooltip.windowFrame:SetTexture("XMacros", "Textures/Tooltip.png")
	tooltip.windowFrame:SetPoint("TOPLEFT", positonFrame, "TOPLEFT", x, y)
	tooltip.windowFrame:SetLayer(parentFrame:GetLayer() + 1)
	tooltip.windowFrame:SetAlpha(0.8)
	tooltip.defaultHeight = tooltip.windowFrame:GetHeight()
	tooltip.defaultWidth = tooltip.windowFrame:GetWidth()
	
	tooltip.wrnLabel = UI.CreateFrame("Text", "WarnLabel", tooltip.windowFrame)
	tooltip.wrnLabel:SetAlpha(1)
	tooltip.wrnLabel:SetPoint("TOPLEFT", tooltip.windowFrame, "TOPLEFT", 10, 10)
	tooltip.wrnLabel:SetPoint("TOPRIGHT", tooltip.windowFrame, "TOPRIGHT", -10, 10)
	tooltip.wrnLabel:SetWordwrap(true)
	tooltip.wrnLabel:SetText("Dummy\n")

	return tooltip
end

function XMacros.Tooltip.ShowTooltip(tooltip, x, y, messages)
	tooltip.callerX = x
	tooltip.callerY = y
	XMacros.Tooltip.SetText(tooltip, messages)
	tooltip.windowFrame:SetVisible(true)
end

function XMacros.Tooltip.HideTooltip(tooltip, x, y)
	if x and y then
		if tooltip.callerX == x and tooltip.callerY == y then
			tooltip.windowFrame:SetVisible(false)
		end
	else
		tooltip.windowFrame:SetVisible(false)
	end
end

function XMacros.Tooltip.SetText(tooltip, messages)
	local message = ""
	for _, msg in ipairs(messages) do
		if message ~= "" then
			message = message .. "\n\n"
		end
		
		message = message .. msg
	end

	tooltip.wrnLabel:SetText(message)
	tooltip.windowFrame:SetHeight(tooltip.wrnLabel:GetHeight() + 20)
end

function XMacros.Tooltip.SetDragEnabled(tooltip, enabled)
	if enabled == false then
		if tooltip.dragFrame then
			tooltip.dragFrame.frame.Event.RightUp = nil
			tooltip.dragFrame.frame.Event.RightUpoutside = nil
			tooltip.dragFrame.frame.Event.RightDown = nil
			tooltip.dragFrame.frame.Event.MouseMove = nil
			tooltip.dragFrame.frame = nil
			tooltip.dragFrame = nil
		end	
	else
		if not tooltip.dragFrame then
			local dragFrame = {}

			dragFrame.dragging = false
			dragFrame.frame = UI.CreateFrame("Frame", "DragFrame", tooltip.windowFrame)
			dragFrame.frame:SetPoint("TOPLEFT", tooltip.windowFrame, "TOPLEFT")
			dragFrame.frame:SetPoint("BOTTOMRIGHT", tooltip.windowFrame, "TOPRIGHT", 0, 50)
			dragFrame.frame:SetLayer(tooltip.windowFrame:GetLayer() + 1)
			dragFrame.frame:SetMouseMasking("full")
			dragFrame.frame.Event.RightUp = function () XMacros.Tooltip.MouseUp(tooltip) end
			dragFrame.frame.Event.RightUpoutside = function () XMacros.Tooltip.MouseUp(tooltip) end
			dragFrame.frame.Event.RightDown = function () XMacros.Tooltip.MouseDown(tooltip) end
			dragFrame.frame.Event.LeftUp = function () XMacros.Tooltip.MouseUp(tooltip) end
			dragFrame.frame.Event.LeftUpoutside = function () XMacros.Tooltip.MouseUp(tooltip) end
			dragFrame.frame.Event.LeftDown = function () XMacros.Tooltip.MouseDown(tooltip) end
			dragFrame.frame.Event.MouseMove = function (event, x, y) XMacros.Tooltip.MouseMove(tooltip, x, y) end
			tooltip.dragFrame = dragFrame
		end
		
		tooltip.wrnLabel:SetText("Drag tooltip to desired location")
		tooltip.windowFrame:SetHeight(tooltip.defaultHeight)
	end
	
	tooltip.windowFrame:SetVisible(enabled)
end

function XMacros.Tooltip.MouseDown(tooltip)
	if tooltip.dragFrame.dragging == false then
		local mouse = Inspect.Mouse()
		tooltip.dragFrame.xOffset = tooltip.x - mouse.x
		tooltip.dragFrame.yOffset = tooltip.y - mouse.y
		tooltip.dragFrame.dragging = true
	end
end

function XMacros.Tooltip.MouseMove(tooltip, x, y)
	if tooltip.dragFrame.dragging == true then
		local totalWidth = UIParent:GetWidth()
		local totalHeight = UIParent:GetHeight()
		tooltip.x = x + tooltip.dragFrame.xOffset
		tooltip.y = y + tooltip.dragFrame.yOffset
		if tooltip.x < 0 then tooltip.x = 0 end
		if tooltip.y < 0 then tooltip.y = 0 end
		if tooltip.x + tooltip.defaultWidth > totalWidth then tooltip.x = totalWidth - tooltip.defaultWidth end
		if tooltip.y + tooltip.defaultHeight > totalHeight then tooltip.y = totalHeight - tooltip.defaultHeight end
		tooltip.windowFrame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", tooltip.x, tooltip.y)
	end
end

function XMacros.Tooltip.MouseUp(tooltip)
	tooltip.dragFrame.dragging = false
	if tooltip.positionChangedCallback then
		tooltip.positionChangedCallback(tooltip, tooltip.x, tooltip.y)
	end
end
